@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: parse.c,v 1.88.2.7 2016/10/18 18:56:52 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: parse.c,v 1.88.2.10 2017/08/10 20:42:29 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - parse.c */
@@ -102,6 +102,10 @@ static int is_builtin_function __PROTO((int t_num));
 
 static void set_up_columnheader_parsing __PROTO((struct at_entry *previous ));
 
+static TBOOLEAN no_iteration __PROTO((t_iterator *));
+static void reevaluate_iteration_limits __PROTO((t_iterator *iter));
+static void reset_iteration __PROTO((t_iterator *iter));
+
 /* Internal variables: */
 
 static struct at_type *at = NULL;
@@ -1106,6 +1110,30 @@ is_builtin_function(int t_num)
     return (0);
 }
 
+/*
+ * Test for the existence of a function without triggering errors
+ * Return values:
+ *   0  no such function is defined
+ *  -1  built-in function
+ *   1  user-defined function
+ */
+int
+is_function(int t_num)
+{
+    struct udft_entry **udf_ptr = &first_udf;
+
+    if (is_builtin_function(t_num))
+	return -1;
+
+    while (*udf_ptr) {
+	if (equals(t_num, (*udf_ptr)->udf_name))
+	    return 1;
+	udf_ptr = &((*udf_ptr)->next_udf);
+    }
+
+    return 0;
+}
+
 /* Look for iterate-over-plot constructs, of the form
  *    for [<var> = <start> : <end> { : <increment>}] ...
  * If one (or more) is found, an iterator structure is allocated and filled
@@ -1182,11 +1210,16 @@ check_for_iteration()
 	    iteration_udv->udv_undef = FALSE;
 	}
 	else if (equals(c_token++, "in")) {
-	    iteration_string = try_to_get_string();
-	    if (!iteration_string)
+	    /* Assume this is a string-valued expression. */
+	    /* It might be worth treating a string constant as a special case */
+	    struct value v;
+	    iteration_start_at = perm_at();
+	    evaluate_at(iteration_start_at, &v);
+	    if (v.type != STRING)
 	    	int_error(c_token-1, errormsg);
 	    if (!equals(c_token++, "]"))
 	    	int_error(c_token-1, errormsg);
+	    iteration_string = v.v.string_val;
 	    iteration_start = 1;
 	    iteration_end = gp_words(iteration_string);
 	    if (iteration_udv->udv_undef == FALSE)
@@ -1226,21 +1259,48 @@ check_for_iteration()
     return iter;
 }
 
+/*
+ * Reevaluate the iteration limits
+ * (in case they are functions whose parameters have taken 
+ * on a new value)
+ */
+static void
+reevaluate_iteration_limits(t_iterator *iter)
+{
+    if (iter->start_at) {
+	struct value v;
+	evaluate_at(iter->start_at, &v);
+	if (iter->iteration_string) {
+	    /* unnecessary if iteration string is a constant */
+	    free(iter->iteration_string);
+	    if (v.type != STRING)
+		int_error(NO_CARET, "corrupt iteration string");
+	    iter->iteration_string = v.v.string_val;
+	    iter->iteration_start = 1;
+	    iter->iteration_end = gp_words(iter->iteration_string);
+	} else {
+	    iter->iteration_start = real(&v);
+	}
+    }
+    if (iter->end_at) {
+	struct value v;
+	evaluate_at(iter->end_at, &v);
+	iter->iteration_end = real(&v);
+    }
+}
+
 /*
  * Reset iteration at this level to start value.
  * Any iteration levels underneath are reset also.
  */
-void
+static void
 reset_iteration(t_iterator *iter)
 {
     if (!iter)
 	return;
 
-    if (iter->start_at) {
-	struct value v;
-	evaluate_at(iter->start_at, &v);
-	iter->iteration_start = real(&v);
-    }
+    reevaluate_iteration_limits(iter);
+    iter->iteration = -1;
     iter->iteration_current = iter->iteration_start;
     if (iter->iteration_string) {
 	gpfree_string(&(iter->iteration_udv->udv_value));
@@ -1251,13 +1311,7 @@ reset_iteration(t_iterator *iter)
 	gpfree_string(&(iter->iteration_udv->udv_value));
 	Ginteger(&(iter->iteration_udv->udv_value), iter->iteration_current);	
     }
-    if (iter->end_at) {
-	struct value v;
-	evaluate_at(iter->end_at, &v);
-	iter->iteration_end = real(&v);
-    }
     reset_iteration(iter->next);
-
 }
 
 /*
@@ -1269,16 +1323,26 @@ TBOOLEAN
 next_iteration(t_iterator *iter)
 {
     /* Once it goes out of range it will stay that way until reset */
-    if (!iter || empty_iteration(iter))
+    if (!iter || no_iteration(iter))
 	return FALSE;
 
     /* Give sub-iterations a chance to advance */
-    if (next_iteration(iter->next))
+    if (next_iteration(iter->next)) {
+	if (iter->iteration < 0)
+	    iter->iteration = 0;
 	return TRUE;
+    }
 
     /* Increment at this level */
-    iter->iteration++;
-    iter->iteration_current += iter->iteration_increment;
+    if (iter->iteration < 0) {
+	/* Just reset, haven't used start value yet */
+	iter->iteration = 0;
+	if (!empty_iteration(iter))
+	    return TRUE;
+    } else {
+	iter->iteration++;
+	iter->iteration_current += iter->iteration_increment;
+    }
     if (iter->iteration_string) {
 	gpfree_string(&(iter->iteration_udv->udv_value));
 	Gstring(&(iter->iteration_udv->udv_value), 
@@ -1295,10 +1359,14 @@ next_iteration(t_iterator *iter)
     if (iter->iteration_increment < 0 &&  iter->iteration_end - iter->iteration_current > 0)
 	return FALSE;
 
+    if (iter->next == NULL)
+	return TRUE;
+
     /* Reset sub-iterations, if any */
     reset_iteration(iter->next);
-    
-    return TRUE;
+
+    /* Go back to top or call self recursively */
+    return next_iteration(iter);
 }
 
 /*
@@ -1306,8 +1374,8 @@ next_iteration(t_iterator *iter)
  * - this really is an iteration and
  * - the top level iteration covers no usable range
  */
-TBOOLEAN
-empty_iteration(t_iterator *iter)
+static TBOOLEAN
+no_iteration(t_iterator *iter)
 {
     if (!iter)
 	return FALSE;
@@ -1317,7 +1385,21 @@ empty_iteration(t_iterator *iter)
 	return TRUE;
     }
 
-    return (empty_iteration(iter->next));
+    return FALSE;
+}
+
+/*
+ * Recursive test that no empty iteration exists in a nested set of iterations
+ */
+TBOOLEAN
+empty_iteration(t_iterator *iter)
+{
+    if (!iter)
+	return FALSE;
+    else if (no_iteration(iter))
+	return TRUE;
+    else
+	return no_iteration(iter->next);
 }
 
 t_iterator *
@@ -1326,8 +1408,8 @@ cleanup_iteration(t_iterator *iter)
     while (iter) {
 	t_iterator *next = iter->next;
 	free(iter->iteration_string);
-	free(iter->start_at);
-	free(iter->end_at);
+	free_at(iter->start_at);
+	free_at(iter->end_at);
 	free(iter);
 	iter = next;
     }
@@ -1370,11 +1452,6 @@ set_up_columnheader_parsing( struct at_entry *previous )
 	    if (at_highest_column_used < u->udv_value.v.int_val)
 		at_highest_column_used = u->udv_value.v.int_val;
 	}
-#if (0) /* Currently handled elsewhere, but could be done here instead */
-	if (u->udv_value.type == STRING) {
-	    parse_1st_row_as_headers = TRUE;
-	}
-#endif
     }
 
     /* NOTE: There is no way to handle ... using (column(<general expression>)) */
