@@ -1,5 +1,5 @@
 /*
- * $Id: setshow.h,v 1.40 2006/06/07 03:54:01 sfeam Exp $
+ * $Id: setshow.h,v 1.50.2.3 2017/05/22 11:54:08 markisch Exp $
  */
 
 /* GNUPLOT - setshow.h */
@@ -44,23 +44,6 @@
 #include "gadgets.h"
 #include "term_api.h"
 
-#define PROGRAM "G N U P L O T"  /* FIXME: move to show.c! */
-
-#define SAVE_NUM_OR_TIME(fp, x, axis)				\
-do{								\
-    if (axis_array[axis].is_timedata) {				\
-	char s[80];						\
-								\
-	putc('"', fp);						\
-	gstrftime(s,80,axis_array[axis].timefmt,(double)(x));	\
-	fputs(conv_text(s), fp);				\
-	putc('"', fp);						\
-    } else {							\
-	fprintf(fp,"%#g",x);					\
-    }								\
-} while(0)
-
-
 /* The set and show commands, in setshow.c */
 void set_command __PROTO((void));
 void unset_command __PROTO((void));
@@ -68,17 +51,25 @@ void reset_command __PROTO((void));
 void show_command __PROTO((void));
 /* and some accessible support functions */
 void show_version __PROTO((FILE *fp));
+void set_format __PROTO((void));
+void set_colorsequence __PROTO((int option));
+void init_special_chars(void);
 char *conv_text __PROTO((const char *s));
-void delete_linestyle __PROTO((struct linestyle_def *, struct linestyle_def *));
+void delete_linestyle __PROTO((struct linestyle_def **, struct linestyle_def *, struct linestyle_def *));
+void delete_dashtype __PROTO((struct custom_dashtype_def *, struct custom_dashtype_def *));
 /* void delete_arrowstyle __PROTO((struct arrowstyle_def *, struct arrowstyle_def *)); */
 void reset_key __PROTO((void));
 void free_marklist __PROTO((struct ticmark * list));
 extern int enable_reset_palette;
 void reset_palette __PROTO((void));
+void rrange_to_xy __PROTO((void));
+void unset_monochrome __PROTO((void));
 
 /* Called from set_label(), plot2d.c and plot3d.c */
-extern void parse_label_options __PROTO((struct text_label *this_label));
+extern void parse_label_options __PROTO((struct text_label *this_label, TBOOLEAN));
 extern struct text_label * new_text_label __PROTO((int tag));
 extern void disp_value __PROTO((FILE *, struct value *, TBOOLEAN));
+extern struct ticmark * prune_dataticks __PROTO((struct ticmark *list));
+
 
 #endif /* GNUPLOT_SETSHOW_H */
