@@ -1,5 +1,5 @@
 /*
- * $Id: wgdiplus.cpp,v 1.16.2.16 2017/03/04 08:20:39 markisch Exp $
+ * $Id: wgdiplus.cpp,v 1.16.2.20 2017/07/03 20:26:31 markisch Exp $
  */
 
 /*
@@ -34,6 +34,7 @@ extern "C" {
 #include <windowsx.h>
 #define GDIPVER 0x0110
 #include <gdiplus.h>
+#include <tchar.h>
 
 #include "wgdiplus.h"
 #include "wgnuplib.h"
@@ -56,6 +57,18 @@ static void gdiplusFilledPolygon(Graphics &graphics, Brush &brush, POINT *ppt, i
 static Brush * gdiplusPatternBrush(int style, COLORREF color, double alpha, COLORREF backcolor, BOOL transparent);
 static void gdiplusDot(Graphics &graphics, Brush &brush, int x, int y);
 static Font * SetFont_gdiplus(Graphics &graphics, LPRECT rect, LPGW lpgw, char * fontname, int size);
+/* Internal state of enhanced text processing.
+*/
+
+static struct {
+	Graphics * graphics; /* graphics object */
+	Font * font;
+	SolidBrush * brush;
+	StringFormat *stringformat;
+} enhstate_gdiplus;
+
+
+/* ****************  ....   ************************* */
 
 
 void
@@ -288,9 +301,7 @@ gdiplusFilledPolygon(Graphics &graphics, Brush &brush, POINT *ppt, int polyi)
 		points[i].X = ppt[i].x;
 		points[i].Y = ppt[i].y;
 	}
-	graphics.SetCompositingQuality(CompositingQualityGammaCorrected);
 	graphics.FillPolygon(&brush, points, polyi);
-	graphics.SetCompositingQuality(CompositingQualityDefault);
 	delete [] points;
 }
 
@@ -363,45 +374,40 @@ SetFont_gdiplus(Graphics &graphics, LPRECT rect, LPGW lpgw, char * fontname, int
 	if ((strikeout = strstr(fontname, " Strikeout")) != NULL)
 		fontStyle |= FontStyleStrikeout;
 	if (italic) *italic = 0;
+	if (bold) *bold = 0;
+	if (underline) *underline = 0;
 	if (strikeout) *strikeout = 0;
-	if (bold) * bold = 0;
-	if (underline) * underline = 0;
 
 	LPWSTR family = UnicodeText(fontname, lpgw->encoding);
-	free(fontname);
 	const FontFamily * fontFamily = new FontFamily(family);
 	free(family);
+	free(fontname);
 	Font * font;
 	int fontHeight;
+	bool deleteFontFamily = true;
 	if (fontFamily->GetLastStatus() != Ok) {
 		delete fontFamily;
 #if (!defined(__MINGW32__) || defined(__MINGW64_VERSION_MAJOR))
 		// MinGW 4.8.1 does not have this
 		fontFamily = FontFamily::GenericSansSerif();
+		deleteFontFamily = false;
 #else
 		family = UnicodeText(GraphDefaultFont(), S_ENC_DEFAULT); // should always be available
 		fontFamily = new FontFamily(family);
 		free(family);
 #endif
-		font = new Font(fontFamily, size * lpgw->sampling, fontStyle, UnitPoint);
-		double scale = font->GetSize() / fontFamily->GetEmHeight(fontStyle) * graphics.GetDpiY() / 72.;
-		/* store text metrics for later use */
-		lpgw->tmHeight = fontHeight = scale * (fontFamily->GetCellAscent(fontStyle) + fontFamily->GetCellDescent(fontStyle));
-		lpgw->tmAscent = scale * fontFamily->GetCellAscent(fontStyle);
-		lpgw->tmDescent = scale * fontFamily->GetCellDescent(fontStyle);
-	} else {
-		font = new Font(fontFamily, size * lpgw->sampling, fontStyle, UnitPoint);
-		double scale = font->GetSize() / fontFamily->GetEmHeight(fontStyle) * graphics.GetDpiY() / 72.;
-		/* store text metrics for later use */
-		lpgw->tmHeight = fontHeight = scale * (fontFamily->GetCellAscent(fontStyle) + fontFamily->GetCellDescent(fontStyle));
-		lpgw->tmAscent = scale * fontFamily->GetCellAscent(fontStyle);
-		lpgw->tmDescent = scale * fontFamily->GetCellDescent(fontStyle);
-		delete fontFamily;
 	}
+	font = new Font(fontFamily, size * lpgw->sampling, fontStyle, UnitPoint);
+	double scale = font->GetSize() / fontFamily->GetEmHeight(fontStyle) * graphics.GetDpiY() / 72.;
+	/* store text metrics for later use */
+	lpgw->tmHeight = fontHeight = scale * (fontFamily->GetCellAscent(fontStyle) + fontFamily->GetCellDescent(fontStyle));
+	lpgw->tmAscent = scale * fontFamily->GetCellAscent(fontStyle);
+	lpgw->tmDescent = scale * fontFamily->GetCellDescent(fontStyle);
+	if (deleteFontFamily)
+		delete fontFamily;
 
 	RectF box;
 	graphics.MeasureString(L"0123456789", -1, font, PointF(0, 0), StringFormat::GenericTypographic(), &box);
-	// lpgw->vchar = MulDiv(box.Height, lpgw->ymax, rect->bottom - rect->top);
 	lpgw->vchar = MulDiv(fontHeight, lpgw->ymax, rect->bottom - rect->top);
 	//lpgw->hchar = MulDiv(box.Width, lpgw->xmax, 10 * (rect->right - rect->left));
 	lpgw->hchar = MulDiv(unsigned(ceil(box.Width)), lpgw->xmax, 10 * (rect->right - rect->left));
@@ -427,6 +433,78 @@ InitFont_gdiplus(LPGW lpgw, HDC hdc, LPRECT rect)
 }
 
 
+static void
+EnhancedSetFont()
+{
+	if ((enhstate.lpgw->fontsize != enhstate.fontsize) ||
+	    (_tcscmp(enhstate.lpgw->fontname, enhstate.fontname) != 0)) {
+		if (enhstate_gdiplus.font)
+			delete enhstate_gdiplus.font;
+		enhstate_gdiplus.font = SetFont_gdiplus(*enhstate_gdiplus.graphics, enhstate.rect, enhstate.lpgw, enhstate.fontname, enhstate.fontsize);
+	}
+}
+
+
+static unsigned
+EnhancedTextLength(char * text)
+{
+	LPWSTR textw = UnicodeText(enhanced_text, enhstate.lpgw->encoding);
+	RectF box;
+	enhstate_gdiplus.graphics->MeasureString(textw, -1, enhstate_gdiplus.font, PointF(0, 0), enhstate_gdiplus.stringformat, &box);
+	free(textw);
+	return ceil(box.Width);
+}
+
+
+static void
+EnhancedPutText(int x, int y, char * text)
+{
+	LPWSTR textw = UnicodeText(text, enhstate.lpgw->encoding);
+	Graphics *g = enhstate_gdiplus.graphics;
+	if (enhstate.lpgw->angle == 0) {
+		PointF pointF(x, y + enhstate.lpgw->tmDescent);
+		g->DrawString(textw, -1, enhstate_gdiplus.font, pointF, enhstate_gdiplus.stringformat, enhstate_gdiplus.brush);
+	} else {
+		/* shift rotated text correctly */
+		g->TranslateTransform(x, y);
+		g->RotateTransform(-enhstate.lpgw->angle);
+		g->DrawString(textw, -1, enhstate_gdiplus.font, PointF(0, enhstate.lpgw->tmDescent), enhstate_gdiplus.stringformat, enhstate_gdiplus.brush);
+		g->ResetTransform();
+	}
+	free(textw);
+}
+
+
+static void
+EnhancedCleanup()
+{
+	delete enhstate_gdiplus.font;
+	delete enhstate_gdiplus.stringformat;
+}
+
+
+static void
+draw_enhanced_init(LPGW lpgw, Graphics &graphics, SolidBrush &brush, LPRECT rect)
+{
+	enhstate.set_font = &EnhancedSetFont;
+	enhstate.text_length = &EnhancedTextLength;
+	enhstate.put_text = &EnhancedPutText;
+	enhstate.cleanup = &EnhancedCleanup;
+
+	enhstate_gdiplus.graphics = &graphics;
+	enhstate_gdiplus.font = SetFont_gdiplus(graphics, rect, lpgw, lpgw->fontname, lpgw->fontsize);
+	enhstate_gdiplus.brush = &brush;
+	enhstate.res_scale = graphics.GetDpiY() / 96.;
+
+	enhstate_gdiplus.stringformat = new StringFormat(StringFormat::GenericTypographic());
+	enhstate_gdiplus.stringformat->SetAlignment(StringAlignmentNear);
+	enhstate_gdiplus.stringformat->SetLineAlignment(StringAlignmentFar);
+	INT flags = enhstate_gdiplus.stringformat->GetFormatFlags();
+	flags |= StringFormatFlagsMeasureTrailingSpaces;
+	enhstate_gdiplus.stringformat->SetFormatFlags(flags);
+}
+
+
 void
 drawgraph_gdiplus(LPGW lpgw, HDC hdc, LPRECT rect)
 {
@@ -547,9 +625,7 @@ drawgraph_gdiplus(LPGW lpgw, HDC hdc, LPRECT rect)
 	Graphics graphics(hdc);
 
 	if (lpgw->antialiasing) {
-		//graphics.SetSmoothingMode(SmoothingModeAntiAlias);
 		graphics.SetSmoothingMode(SmoothingModeAntiAlias8x8);
-		// graphics.SetTextRenderingHint(TextRenderingHintAntiAlias);
 		graphics.SetTextRenderingHint(TextRenderingHintClearTypeGridFit);
 	}
 	graphics.SetInterpolationMode(InterpolationModeNearestNeighbor);
@@ -589,6 +665,9 @@ drawgraph_gdiplus(LPGW lpgw, HDC hdc, LPRECT rect)
 	StringFormat stringFormat(StringFormat::GenericTypographic());
 	stringFormat.SetAlignment(StringAlignmentNear);
 	stringFormat.SetLineAlignment(StringAlignmentNear);
+	INT flags = stringFormat.GetFormatFlags();
+	flags |= StringFormatFlagsMeasureTrailingSpaces;
+	stringFormat.SetFormatFlags(flags);
 	font = SetFont_gdiplus(graphics, rect, lpgw, NULL, 0);
 
 	/* calculate text shifting for horizontal text */
@@ -624,8 +703,8 @@ drawgraph_gdiplus(LPGW lpgw, HDC hdc, LPRECT rect)
 
 	while (ngwop < lpgw->nGWOP) {
 		/* transform the coordinates */
-		xdash = MulDiv(curptr->x, rr - rl, lpgw->xmax) + rl;
-		ydash = MulDiv(curptr->y, rt - rb, lpgw->ymax) + rb - 1;
+		xdash = MulDiv(curptr->x, rr - rl - 1, lpgw->xmax) + rl;
+		ydash = rb - MulDiv(curptr->y, rb - rt - 1, lpgw->ymax) + rt - 1;
 
 		/* ignore superfluous moves - see bug #1523 */
 		/* FIXME: we should do this in win.trm, not here */
@@ -818,7 +897,7 @@ drawgraph_gdiplus(LPGW lpgw, HDC hdc, LPRECT rect)
 			pen.SetColor(color);
 			pen.SetWidth(cur_penstruct.lopnWidth.x);
 			if (cur_penstruct.lopnStyle <= PS_DASHDOTDOT)
-				// cast is save since GDI and GDI+ use the same numbers
+				// cast is safe since GDI and GDI+ use the same numbers
 				gdiplusSetDashStyle(&pen, static_cast<DashStyle>(cur_penstruct.lopnStyle));
 			else
 				pen.SetDashStyle(DashStyleSolid);
@@ -875,7 +954,6 @@ drawgraph_gdiplus(LPGW lpgw, HDC hdc, LPRECT rect)
 						graphics.DrawString(textw, -1, font, PointF(0,0), &stringFormat, &solid_brush);
 						graphics.ResetTransform();
 					}
-					// only need to measure the string when boxing or drawing a key text
 					RectF size;
 					int dxl, dxr;
 #ifndef EAM_BOXED_TEXT
@@ -883,7 +961,7 @@ drawgraph_gdiplus(LPGW lpgw, HDC hdc, LPRECT rect)
 #else
 					if (keysample || boxedtext.boxing) {
 #endif
-						graphics.MeasureString(textw, -1, font, PointF(0,0), StringFormat::GenericTypographic(), &size);
+						graphics.MeasureString(textw, -1, font, PointF(0,0), &stringFormat, &size);
 						if (lpgw->justify == LEFT) {
 							dxl = 0;
 							dxr = size.Width;
@@ -920,24 +998,11 @@ drawgraph_gdiplus(LPGW lpgw, HDC hdc, LPRECT rect)
 		}
 
 		case W_enhanced_text: {
-			/* TODO: This section still uses GDI. Convert to GDI+. */
-			HDC hdc = graphics.GetHDC();
-			char * str;
-			str = (char *) LocalLock(curptr->htext);
+			char * str = (char *) LocalLock(curptr->htext);
 			if (str) {
 				RECT extend;
-
-				/* Setup GDI fonts: force re-make */
-				int save_fontsize = lpgw->fontsize;
-				lpgw->fontsize = -1;
-				GraphChangeFont(lpgw, lpgw->fontname, save_fontsize, hdc, *rect);
-				SetFont(lpgw, hdc);
-				lpgw->fontsize = save_fontsize;
-
-				/* Set GDI text color */
-				SetTextColor(hdc, last_color);
-
-				draw_enhanced_text(lpgw, hdc, rect, xdash, ydash, str);
+				draw_enhanced_init(lpgw, graphics, solid_brush, rect);
+				draw_enhanced_text(lpgw, rect, xdash, ydash, str);
 				draw_get_enhanced_text_extend(&extend);
 
 				if (keysample) {
@@ -960,7 +1025,6 @@ drawgraph_gdiplus(LPGW lpgw, HDC hdc, LPRECT rect)
 #endif
 			}
 			LocalUnlock(curptr->htext);
-			graphics.ReleaseHDC(hdc);
 			break;
 		}
 
@@ -1141,7 +1205,8 @@ drawgraph_gdiplus(LPGW lpgw, HDC hdc, LPRECT rect)
 					/* style == 2 --> use fill pattern according to
 							 * fillpattern. Pattern number is enumerated */
 					int pattern = GPMAX(fillstyle >> 4, 0) % pattern_num;
-					if (pattern_brush) delete pattern_brush;
+					if (pattern_brush)
+						delete pattern_brush;
 					pattern_brush = gdiplusPatternBrush(pattern,
 									last_color, 1., lpgw->background, transparent);
 					fill_brush = pattern_brush;
@@ -1195,6 +1260,13 @@ drawgraph_gdiplus(LPGW lpgw, HDC hdc, LPRECT rect)
 				/* recalculate shifting of rotated text */
 				hshift = - sin(M_PI/180. * lpgw->angle) * lpgw->tmHeight / 2;
 				vshift = - cos(M_PI/180. * lpgw->angle) * lpgw->tmHeight / 2;
+				if (lpgw->antialiasing) {
+					// Cleartype is only applied to non-rotated text
+					if ((lpgw->angle % 180) != 0)
+						graphics.SetTextRenderingHint(TextRenderingHintAntiAliasGridFit);
+					else
+						graphics.SetTextRenderingHint(TextRenderingHintClearTypeGridFit);
+				}
 			}
 			break;
 
@@ -1220,16 +1292,16 @@ drawgraph_gdiplus(LPGW lpgw, HDC hdc, LPRECT rect)
 			font = SetFont_gdiplus(graphics, rect, lpgw, fontname, size);
 			LocalUnlock(curptr->htext);
 			/* recalculate shifting of rotated text */
-			hshift = - sin(M_PI/180. * lpgw->angle) * lpgw->tmHeight / 2;
-			vshift = - cos(M_PI/180. * lpgw->angle) * lpgw->tmHeight / 2;
+			hshift = - sin(M_PI / 180. * lpgw->angle) * lpgw->tmHeight / 2.;
+			vshift = - cos(M_PI / 180. * lpgw->angle) * lpgw->tmHeight / 2.;
 			break;
 		}
 
 		case W_pointsize:
 			if (curptr->x > 0) {
 				double pointsize = curptr->x / 100.0;
-				htic = pointsize * MulDiv(lpgw->htic, rr-rl, lpgw->xmax) + 1;
-				vtic = pointsize * MulDiv(lpgw->vtic, rb-rt, lpgw->ymax) + 1;
+				htic = MulDiv(pointsize * lpgw->htic, rr - rl, lpgw->xmax) + 1;
+				vtic = MulDiv(pointsize * lpgw->vtic, rb - rt, lpgw->ymax) + 1;
 			} else {
 				htic = vtic = 0;
 			}
@@ -1468,12 +1540,12 @@ drawgraph_gdiplus(LPGW lpgw, HDC hdc, LPRECT rect)
 			// Switch between cached and direct drawing
 			if (ps_caching) {
 				// Create a compatible bitmap
-				b = new Bitmap(2*htic+3, 2*vtic+3);
+				b = new Bitmap(2 * htic + 3, 2 * vtic + 3, &graphics);
 				g = Graphics::FromImage(b);
 				if (lpgw->antialiasing)
 					g->SetSmoothingMode(SmoothingModeAntiAlias8x8);
-				cb_ofs.x = xofs = htic+1;
-				cb_ofs.y = yofs = vtic+1;
+				cb_ofs.x = xofs = htic + 1;
+				cb_ofs.y = yofs = vtic + 1;
 				last_symbol = curptr->op;
 			} else {
 				g = &graphics;
@@ -1489,7 +1561,8 @@ drawgraph_gdiplus(LPGW lpgw, HDC hdc, LPRECT rect)
 			case W_star: /* do star: first plus, then cross */
 				g->DrawLine(&solid_pen, xofs - htic, yofs, xofs + htic, yofs);
 				g->DrawLine(&solid_pen, xofs, yofs - vtic, xofs, yofs + vtic);
-				if (curptr->op == W_plus) break;
+				if (curptr->op == W_plus)
+					break;
 			case W_cross: /* do X */
 				g->DrawLine(&solid_pen, xofs - htic, yofs - vtic, xofs + htic - 1, yofs + vtic);
 				g->DrawLine(&solid_pen, xofs - htic, yofs + vtic, xofs + htic - 1, yofs - vtic);
